
# $Id: DACScheck.pm.in 2529 2011-09-23 22:19:51Z brachman $
# Copyright (c) 2003-2011
# Distributed Systems Software.  All rights reserved.
# See the file LICENSE for redistribution information.

# A crude interface between Perl and dacscheck
# Initialize by calling:
#   1) dacscheck_prog to set the absolute pathname of the dacscheck command
#   2) dacscheck_rules to set the absolute pathname of the root directory
#      containing the DACS rules (if the default is incorrect or rules are not
#      specified with each check).
#
# Then, to check for authorization, call:
#    dacscheck(id, resource, ...);
# or from the CGI environment:
#    dacscheck_cgi(resource, ...);
#
# These functions return 1 if access is granted, 0 if denied, or -1 if
# an error occurs.
#
# Callers must do:
#    use DACScheck;
#
# See dacscheck(1)

package DACScheck;

use strict;

require Exporter;
our @ISA = qw(Exporter);

# Export symbols

our @EXPORT = qw(dacscheck dacscheck_cgi dacscheck_prog dacscheck_rules);
our $VERSION = '1.0.0';

# Locals

my $dacscheck_prog = "/usr/bin/dacscheck";
my $dacscheck_rules = "";

# Return something

1;

# Set the path to use for the dacscheck command
#
sub dacscheck_prog {
   my ($prog) = @_;
   my $oldprog;

   $oldprog = $dacscheck_prog;
   $dacscheck_prog = $prog;

   return $oldprog;
}

# Set the path to use for the dacscheck rules
#
sub dacscheck_rules {

   my ($rules) = @_;

   my $oldrules;

   $oldrules = $dacscheck_rules;
   $dacscheck_rules = $rules;

   return $oldrules;
}

# Internal function to actually run dacscheck
#
sub run_dacscheck {
  my $exit_value;
  my $result;

  system @_;
  $exit_value  = $? >> 8;

  # print "dacscheck returned $exit_value for user \"$remote_user\"\n";
  if ($exit_value == 1) {
      # dacscheck denies access
      $result = 0;
   }
   elsif ($exit_value == 0) {
      # dacscheck grants access
      $result = 1;
   }
   else {
      # dacscheck error occurred
      $result = -1;
   }

   return $result;
}

# A simple interface to the dacscheck command
# Check if $id is granted access to $resource; additional arguments may
# follow
#
sub dacscheck {

   my ($id, $resource, @other_args) = @_;
   my @args;

   if ($dacscheck_prog eq "") {
     return -1;
   }

   if ($dacscheck_rules eq "") {
     @args = ($dacscheck_prog, '-q', '-i', $id, @other_args, $resource);
   }
   else {
     @args = ($dacscheck_prog, '-q', '-i', $id, '-rules', $dacscheck_rules,
	    @other_args, $resource);
   }

   return run_dacscheck(@args);
}

# A simple interface to the dacscheck command
# Check if access to $resource is granted based on REMOTE_USER; additional
# arguments may follow
# Return 1 if access is granted
# Return 0 if access is denied
# Return -1 if an error occurs
#
sub dacscheck_cgi {

   my ($resource, @other_args) = @_;
   my @args;

   if ($dacscheck_prog eq "") {
     return -1;
   }

   if ($dacscheck_rules eq "") {
     @args = ($dacscheck_prog, '-q', '-icgi', @other_args, $resource);
   }
   else {
     @args = ($dacscheck_prog, '-q', '-icgi', '-rules', $dacscheck_rules,
         @other_args, $resource);
   }

   return run_dacscheck(@args);
}

